@@ -1,6 +1,6 @@
-/* vobcopy 1.1.2
+/* vobcopy 1.2.0
  *
- * Copyright (c) 2001 - 2007 robos@muon.de
+ * Copyright (c) 2001 - 2009 robos@muon.de
  * Lots of contribution and cleanup from rosenauer@users.sourceforge.net
  * Critical bug-fix from Bas van den Heuvel
  * Takeshi HIYAMA made lots of changes to get it to run on FreeBSD
@@ -18,8 +18,7 @@
  *  GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with vobcopy; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with vobcopy; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /* CONTRIBUTORS (direct or through source "borrowing")
@@ -75,7 +74,7 @@ and potentially fatal."  - Thanks Leigh!*/
   char              pwd[255],provided_output_dir[255],provided_input_dir[255];
   char              alternate_output_dir[4][255], onefile[255];
   unsigned char     bufferin[ DVD_VIDEO_LB_LEN * BLOCK_COUNT ];
-  int               i = 0,j = 0, l = 0, argc_i = 0, alternate_dir_count = 0;
+  int               i = 0,j = 0, argc_i = 0, alternate_dir_count = 0;
   int               partcount = 0, get_dvd_name_return, options_char = 0;
   int               dvd_count = 0, verbosity_level = 0, paths_taken = 0, fast_factor = 1;
   int               watchdog_minutes = 0;
@@ -92,8 +91,10 @@ and potentially fatal."  - Thanks Leigh!*/
   bool              mirror_flag = FALSE, provided_dvd_name_flag = FALSE;
   bool              stdout_flag = FALSE, space_greater_2gb_flag = TRUE;
   bool              fast_switch = FALSE, onefile_flag = FALSE;
-  bool              quiet_flag = FALSE;
+  bool              quiet_flag = FALSE, longest_title_flag = FALSE;
   struct stat       buf;
+  float             lastpos = 0;
+  int               starttime;
 
   dvd_reader_t      *dvd = NULL;
   dvd_file_t        *dvd_file = NULL;
@@ -140,6 +141,7 @@ and potentially fatal."  - Thanks Leigh!*/
         {"input-dir", 1, 0, 'i'},
         {"info", 0, 0, 'I'},
         {"large-file", 0, 0, 'l'},
+        {"longest", 0, 0, 'M'},
         {"mirror", 0, 0, 'm'},
         {"title-number", 1, 0, 'n'},
         {"output-dir", 1, 0, 'o'},
@@ -171,10 +173,10 @@ and potentially fatal."  - Thanks Leigh!*/
     {
 #ifdef HAVE_GETOPT_LONG
       options_char = getopt_long( argc, argv,
-                                  "1:2:3:4:a:b:c:e:i:n:o:qO:t:vfF:lmhL:Vw:Ix",
+                                  "1:2:3:4:a:b:c:e:i:n:o:qO:t:vfF:lmMhL:Vw:Ix",
                                   long_options ,&option_index);
 #else
-      options_char = getopt( argc, argv, "1:2:3:4:a:b:c:e:i:n:o:qO:t:vfF:lmhL:Vw:Ix-" );
+      options_char = getopt( argc, argv, "1:2:3:4:a:b:c:e:i:n:o:qO:t:vfF:lmMhL:Vw:Ix-" );
 #endif
 
       if ( options_char == -1 ) break;
@@ -333,6 +335,11 @@ and potentially fatal."  - Thanks Leigh!*/
           info_flag = TRUE;
           break;
 
+        case'M':/*Main track - i.e. the longest track on the dvd*/
+          titleid_flag = TRUE;
+	  longest_title_flag = TRUE;
+          break;
+
         case'n': /*title number*/
           if ( !isdigit( (int) *optarg ) )
             {
@@ -397,6 +404,12 @@ and potentially fatal."  - Thanks Leigh!*/
               stdout_flag = TRUE;
               force_flag = TRUE;
             }
+	  for( i=0; i< strlen(provided_dvd_name);i++ )
+	    {
+	      if( provided_dvd_name[i] == ' ')
+		provided_dvd_name[i] = '_';
+	    }
+
           break;
 
         case'v': /*verbosity level, can be called multiple times*/
@@ -475,7 +488,7 @@ and potentially fatal."  - Thanks Leigh!*/
           i = 0;
           break;
         case'V': /*version number output */
-          printf( "Vobcopy "VERSION" - GPL Copyright (c) 2001 - 2007 robos@muon.de\n" );
+          printf( "Vobcopy "VERSION" - GPL Copyright (c) 2001 - 2009 robos@muon.de\n" );
           exit( 0 );
           break;
 
@@ -500,7 +513,7 @@ and potentially fatal."  - Thanks Leigh!*/
         }
     }
 
-  fprintf( stderr, _("Vobcopy "VERSION" - GPL Copyright (c) 2001 - 2007 robos@muon.de\n") );
+  fprintf( stderr, _("Vobcopy "VERSION" - GPL Copyright (c) 2001 - 2009 robos@muon.de\n") );
   fprintf( stderr, _("[Hint] All lines starting with \"libdvdread:\" are not from vobcopy but from the libdvdread-library\n") );
 
   /*get the current working directory*/
@@ -520,6 +533,7 @@ and potentially fatal."  - Thanks Leigh!*/
 
   add_end_slash( pwd );
 
+
   if( quiet_flag )
     {
       int temp;
@@ -530,13 +544,19 @@ and potentially fatal."  - Thanks Leigh!*/
       fprintf( stderr, _("[Hint] Quiet mode - All messages will now end up in %s\n"), tmp_path );
       if ( ( temp = open( tmp_path , O_RDWR | O_CREAT | O_EXCL, 0666 ) ) == -1 )
 	{
-	  printf( "[Error] Error: %s\n", strerror( errno ) );
 	  if ( errno == EEXIST )
 	    {
-	      printf( "\n[Error] The file %s already exists. Since overwriting it can be seen as a security problem I stop here.\n", tmp_path );
-	      printf( "[Hint] Use -f to override or simply delete that file\n" );
 	      if ( !force_flag )
-		exit( 1 );
+		{
+		  printf( "[Error] Error: %s\n", strerror( errno ) );
+		  printf( "\n[Error] The file %s already exists. Since overwriting it can be seen as a security problem I stop here.\n", tmp_path );
+		  printf( "[Hint] Use -f to override or simply delete that file\n" );
+		  exit( 1 );
+		}
+	      else
+		{
+		  printf( "[Warning] Overwriting %s as requested with -f\n", tmp_path );
+		}
 	    }
 	  else
 	    {
@@ -667,7 +687,8 @@ and potentially fatal."  - Thanks Leigh!*/
       if ( ( result = get_device( provided_input_dir, dvd_path ) ) < 0 )
         {
           fprintf( stderr, _("[Error] Could not get the device which belongs to the given path!\n") );
-          exit( 1 );
+          fprintf( stderr, _("[Hint] Will try to open it as a directory/image file\n") );
+	  /*           exit( 1 ); */
         }
       if (result == 0)
         mounted = FALSE;
@@ -679,11 +700,20 @@ and potentially fatal."  - Thanks Leigh!*/
     {
       if ( ( dvd_count = get_device_on_your_own( provided_input_dir, dvd_path ) ) <= 0 )
         {
-          fprintf( stderr, _("[Error] Could not get the device and path! Maybe not mounted the dvd?\n") );
-          exit( 1 );
+          fprintf( stderr, _("[Warning] Could not get the device and path! Maybe not mounted the dvd?\n") );
+          fprintf( stderr, _("[Hint] Will try to open it as a directory/image file\n") );
+	  /*           exit( 1 ); */
         }
       if( dvd_count > 0 )
         mounted = TRUE;
+      else
+        mounted = FALSE;
+    }
+
+  if (! mounted )
+    {
+      /*see if the path given is a iso file or a VIDEO_TS dir */
+      safestrncpy( dvd_path, provided_input_dir, sizeof(dvd_path)-1 );
     }
 
   /*
@@ -704,15 +734,11 @@ and potentially fatal."  - Thanks Leigh!*/
   /*
    * this here gets the dvd name
    */
-  if( !provided_dvd_name_flag )
-    {
-      get_dvd_name_return = get_dvd_name( dvd_path, dvd_name );
-      fprintf( stderr, _("[Info] Name of the dvd: %s\n"), dvd_name );
-    }
+  if ( provided_dvd_name_flag )
+    safestrncpy( dvd_name, provided_dvd_name, sizeof(dvd_name)-1 );
   else
-    {
-      safestrncpy( dvd_name, provided_dvd_name, sizeof(dvd_name)-1 );
-    }
+    get_dvd_name_return = get_dvd_name( dvd_path, dvd_name );
+  fprintf( stderr, _("[Info] Name of the dvd: %s\n"), dvd_name );
 
   /* The new part taken from play-title.c*/
 
@@ -745,6 +771,11 @@ and potentially fatal."  - Thanks Leigh!*/
 
     }
 
+  if( longest_title_flag ) /*no title specified (-n ) */
+    {
+      titleid = get_longest_title( &dvd );
+      fprintf( stderr, _("[Info] longest title %d.\n"), titleid );
+    }
 
   if( !titleid_flag ) /*no title specified (-n ) */
     {
@@ -886,6 +917,11 @@ and potentially fatal."  - Thanks Leigh!*/
   if( mirror_flag )
     {/*mirror beginning*/
       fprintf( stderr, _("\n[Info] DVD-name: %s\n"), dvd_name );
+      if( provided_dvd_name_flag )
+	{
+	  fprintf( stderr, _("\n[Info] Your name for the dvd: %s\n"), provided_dvd_name );
+	  safestrncpy( dvd_name, provided_dvd_name, sizeof(dvd_name)-1 );
+	}
       fprintf( stderr, _("[Info]  Disk free: %.0f MB\n"), (float) pwd_free / ( 1024*1024 ) );
       fprintf( stderr, _("[Info]  Vobs size: %.0f MB\n"), (float) disk_vob_size / ( 1024*1024 ) );
       if( ( force_flag || pwd_free > disk_vob_size ) && alternate_dir_count < 2 )
@@ -897,7 +933,7 @@ and potentially fatal."  - Thanks Leigh!*/
           char output_file[255];
           int  i, start, title_nr = 0;
           off_t file_size;
-          double percent = 0, tmp_i = 0, tmp_file_size = 0;
+          double tmp_i = 0, tmp_file_size = 0;
           int k = 0;
           char d_name[256];
 
@@ -1087,8 +1123,8 @@ next: /*for the goto - ugly, I know... */
                                   fprintf( stderr, _("[Error] Error: %s\n"), strerror( errno ) );
                                   exit ( 1 );
                                 }
-                              else
-                                close( streamout );
+			      /*                              else
+							      close( streamout ); */
                               overwrite_flag = TRUE;
 			      if ( op == 'x' )
 				{
@@ -1218,7 +1254,6 @@ next: /*for the goto - ugly, I know... */
               if( strstr( directory->d_name, ".vob" )
                   || strstr( directory->d_name, ".VOB"  ) )
                 {
-                  int l=0;
                   if( directory->d_name[7] == 48 || title_nr == 0  )
                     {
                       /*this is vts_xx_0.vob or video_ts.vob, a menu vob*/
@@ -1273,6 +1308,7 @@ next: /*for the goto - ugly, I know... */
                   if( verbosity_level > 1)
                     fprintf( stderr, _("[Info] Start of %s at %d blocks \n"), output_file, start );
                   file_block_count = block_count;
+		  starttime = time(NULL);
                   for( i = start; ( i - start ) * DVD_VIDEO_LB_LEN < file_size; i += file_block_count)
                     {
 		      int tries = 0, skipped_blocks = 0; 
@@ -1310,30 +1346,32 @@ next: /*for the goto - ugly, I know... */
                           exit( 1 );
                         }
 
-                      l += blocks;
                       /*progression bar*/
                       /*this here doesn't work with -F 10 */
                       /*		      if( !( ( ( ( i-start )+1 )*DVD_VIDEO_LB_LEN )%( 1024*1024 ) ) ) */
-                      if( l > 100 )
+		      progressUpdate(starttime, (int)(( ( i-start+1 )*DVD_VIDEO_LB_LEN )), (int)(tmp_file_size+2048), FALSE);
+		      /*
+                      if( check_progress() )
                         {
                           tmp_i = ( i-start );
 
                           percent = ( ( ( ( tmp_i+1 )*DVD_VIDEO_LB_LEN )*100 )/tmp_file_size );
-                          fprintf( stderr, _("%4.0fMB of %4.0fMB written "),
+                          fprintf( stderr, _("\r%4.0fMB of %4.0fMB written "),
                                    ( ( tmp_i+1 )*DVD_VIDEO_LB_LEN )/( 1024*1024 ),
                                    ( tmp_file_size+2048 )/( 1024*1024 ) );
-                          fprintf( stderr, _("( %3.1f %% ) \r"), percent );
-                          l = 0;
+                          fprintf( stderr, _("( %3.1f %% ) "), percent );
                         }
-
+		      */
                     }
 /*this is just so that at the end it actually says 100.0% all the time... */
 /*TODO: if it is correct to always assume it's 100% is a good question.... */
-                  fprintf( stderr, _("%4.0fMB of %4.0fMB written "),
+/*                  fprintf( stderr, _("\r%4.0fMB of %4.0fMB written "),
                            ( ( tmp_i+1 )*DVD_VIDEO_LB_LEN )/( 1024*1024 ),
                            ( tmp_file_size+2048 )/( 1024*1024 ) );
-                  fprintf( stderr, _("( 100.0%% ) \r") );
-
+                  fprintf( stderr, _("( 100.0%% ) ") );
+*/
+		  lastpos = 0;
+		  progressUpdate(starttime, (int)(( ( i-start+1 )*DVD_VIDEO_LB_LEN )), (int)(tmp_file_size+2048), TRUE);
                   start=i;
                   fprintf( stderr, _("\n") );
                   if( !stdout_flag )
@@ -1453,13 +1491,13 @@ next: /*for the goto - ugly, I know... */
   if ( mounted && verbose_flag )
     {
       fprintf( stderr, _("\n[Info] Difference between vobsize read from cd and size returned from libdvdread:\n") );
-      /*        fprintf( stderr, _("vob_size (stat) = %lu\nlibdvdsize      = %lu\ndiff            = %lu\n"),  //TODO:the diff returns only crap...
+      /*        fprintf( stderr, _("vob_size (stat) = %lu\nlibdvdsize      = %lu\ndiff            = %lu\n"),  TODO:the diff returns only crap...
                       vob_size, 
                       ( off_t ) ( file_size_in_blocks ) * ( off_t ) DVD_VIDEO_LB_LEN, 
                       ( off_t ) vob_size - ( off_t ) ( ( off_t )( file_size_in_blocks ) * ( off_t ) ( DVD_VIDEO_LB_LEN ) ) ); */
       fprintf( stderr, _("[Info] Vob_size (stat) = %lu\n[Info] libdvdsize      = %lu\n"),
-               vob_size,
-               ( off_t ) ( file_size_in_blocks ) * ( off_t ) DVD_VIDEO_LB_LEN );
+               (long unsigned int) vob_size,
+               (long unsigned int)(( off_t ) ( file_size_in_blocks ) * ( off_t ) DVD_VIDEO_LB_LEN ));
     }
 
   if( info_flag )
@@ -1522,13 +1560,16 @@ next: /*for the goto - ugly, I know... */
     }
 
   /*
-from bash.org
+from alt.tasteless.jokes:
 
-#464444 +(878)- [X]
+A farmer walks into his bedroom with a sheep under his arm.
+"Honey, this pig is what I fuck when you don't feel well."
+
+The wife looks up from the magazine she's reading and says,
+"That's not a pig, it's a sheep."
+
+The man replies, "I was talking to the sheep."
 
-<Phryss> Sometimes, I sit back and think about what my father used to tell 
-me about the birds and the bees: "Stop fucking the dog.  The neighbors are 
-watching, and it's their dog."
   */
 
   /*********************
@@ -1539,7 +1580,7 @@ watching, and it's their dog."
     /*if the user has given a name for the file */
     {
       fprintf( stderr, _("\n[Info] Your name for the dvd: %s\n"), provided_dvd_name );
-      strcpy( dvd_name, provided_dvd_name );
+      safestrncpy( dvd_name, provided_dvd_name, sizeof(dvd_name)-1 );
     }
 
   while( offset < ( file_size_in_blocks - seek_start - stop_before_end ) )
@@ -1562,19 +1603,6 @@ watching, and it's their dog."
             }
           else
             {
-              /* 	      if( alternate_dir_count == 1 && !space_greater_2gb_flag )  */
-              /*shouldn't come to this only if some data is still there but
-                no free space left... */
-              /* 		{ */
-              /* 		  fprintf( stderr,  */
-              /* 			   "\nHmm, looks like there wasn't enough space in the provided dirs ...\n" */
-              /* 			   "Did you use -1, -2, -3 or even up to -4 to specify alternate dirs?\n" ); */
-              /* 		  exit( 1 ); */
-              /* 		} */
-              /* 	      for( i = 1; i < 5; i++ ) */
-              /* 		{ */
-              /* 		  if( paths_taken == i && alternate_dir_count > 1 ) */
-
               for( i = 1; i < alternate_dir_count; i++ )
                 {
                   if( paths_taken == i )
@@ -1700,8 +1728,8 @@ watching, and it's their dog."
                           fprintf( stderr, _("\n[Error] Error opening file %s\n"), name );
                           exit ( 1 );
                         }
-                      else
-                        close (streamout);
+		      /*                      else
+					      close (streamout); */
                       overwrite_flag = TRUE;
 		      if ( op == 'x' )
 			{
@@ -1720,8 +1748,8 @@ watching, and it's their dog."
                           fprintf( stderr, _("\n[Error] Error opening file %s\n"), name );
                           exit ( 1 );
                         }
-                      else
-                        close( streamout );
+		      /*                      else
+					      close( streamout ); */
                       if( verbosity_level >= 1 )
                         fprintf( stderr, _("[Info] User chose append\n") );
                       break;
@@ -1763,16 +1791,13 @@ watching, and it's their dog."
       fprintf( stderr, _("\n") );
       memset( bufferin, 0, BLOCK_COUNT * DVD_VIDEO_LB_LEN * sizeof( unsigned char ) );
 
-      /*       for ( ; ( offset + ( off_t ) seek_start ) < ( ( off_t ) file_size_in_blocks - ( off_t ) stop_before_end )   */
-      /* 	     && offset - ( off_t )max_filesize_in_blocks_summed < max_filesize_in_blocks;  */
-      /* 	    offset++ ) */
-      /* 	{ */
-
       file_block_count = block_count;
+      starttime = time(NULL);
       for ( ; ( offset + ( off_t ) seek_start ) < ( ( off_t ) file_size_in_blocks - ( off_t ) stop_before_end )
             && offset - ( off_t )max_filesize_in_blocks_summed - (off_t)angle_blocks_skipped < max_filesize_in_blocks;
             offset += file_block_count )
         {
+	  int tries = 0, skipped_blocks = 0; 
           /* Only read and write as many blocks as there are left in the file */
           if ( ( offset + file_block_count + ( off_t ) seek_start ) > ( ( off_t ) file_size_in_blocks - ( off_t ) stop_before_end ) )
             {
@@ -1783,7 +1808,28 @@ watching, and it's their dog."
               file_block_count = max_filesize_in_blocks - ( offset + file_block_count - ( off_t )max_filesize_in_blocks_summed - (off_t)angle_blocks_skipped );
             }
 
-          blocks = DVDReadBlocks( dvd_file,( offset + seek_start ), file_block_count, bufferin );
+	  /*          blocks = DVDReadBlocks( dvd_file,( offset + seek_start ), file_block_count, bufferin ); */
+
+	  while( ( blocks = DVDReadBlocks( dvd_file,( offset + seek_start ), file_block_count, bufferin ) ) <= 0 && tries < 10 )
+	    {
+	      if( tries == 9 )
+		{
+		  offset += file_block_count;
+		  skipped_blocks +=1;
+		  overall_skipped_blocks +=1;
+		  tries=0;
+		}
+	      /*                          if( verbosity_level >= 1 ) 
+					  fprintf( stderr, _("[Warn] Had to skip %d blocks (reading block %d)! \n "), skipped_blocks, i ); */
+	      tries++;
+	    }
+	  
+	  if( verbosity_level >= 1 && skipped_blocks > 0 )
+	    fprintf( stderr, _("[Warn] Had to skip (couldn't read) %d blocks (before block %d)! \n "), skipped_blocks, offset );
+
+/*TODO: this skipping here writes too few bytes to the output */
+
+
           if( write( streamout, bufferin, DVD_VIDEO_LB_LEN * blocks ) < 0 )
             {
               fprintf( stderr, _("\n[Error] Write() error\n") );
@@ -1793,21 +1839,11 @@ watching, and it's their dog."
               fprintf( stderr, _("[Error] Error: %s\n"), strerror( errno ) );
               exit( 1 );
             }
-          l += blocks;
 
           /*this is for people who report that it takes vobcopy ages to copy something */
           /* TODO */
-
-
-          if( l > 100 )
-            {
-              /*this is the progress indicator*/
-              fprintf( stderr, _("%4.0fMB of %4.0fMB written (%.0f %%)\r"),
-                       ( float ) offset/512,
-                       ( float ) ( file_size_in_blocks - seek_start - stop_before_end )/512 ,
-                       ( float ) ( offset*100 )/( file_size_in_blocks - seek_start - stop_before_end ) );
-              l=0;
-            }
+	  
+	  progressUpdate(starttime, (int)offset/512, (int)( file_size_in_blocks - seek_start - stop_before_end )/512, FALSE);
         }
       if( !stdout_flag )
         {
@@ -1817,10 +1853,7 @@ watching, and it's their dog."
               fprintf( stderr, _("[Error] error: %s\n"), strerror( errno ) );
               exit( 1 );
             }
-	  /*this is just so that it says 100% at the end. Dirty.... */
-          fprintf( stderr, _("%4.0fMB of %4.0fMB written ( 100.0 %%)\r"),
-                   ( float ) offset/512,
-                   ( float ) ( file_size_in_blocks - seek_start - stop_before_end )/512 );
+	  progressUpdate(starttime, (int)offset/512, (int)( file_size_in_blocks - seek_start - stop_before_end )/512, TRUE);
 
           close( streamout );
 
@@ -1849,15 +1882,6 @@ watching, and it's their dog."
           else if( !cut_flag )
             {
               re_name( name );
-              /* TODO: this check should go in again
-              if( ( ( 1048571 * 2048 ) - disk_vob_size ) < MAX_DIFFER )
-              {
-              re_name( name );
-              }
-              else
-              {
-              fprintf( stderr, _("\n[Error]File size (%.0f) of %s differs largely from 2GB, therefore keeps it's .partial\n"), ( float ) buf.st_size, name );
-              } */
             }
 
           else if( cut_flag )
@@ -1892,7 +1916,6 @@ watching, and it's their dog."
   fprintf( stderr, _("\n[Info] Copying finished! Let's see if the sizes match (roughly)\n") );
   fprintf( stderr, _("[Info] Combined size of title-vobs: %.0f (%.0f MB)\n"), ( float ) vob_size, ( float ) vob_size / ( 1024*1024 ) );
   fprintf( stderr, _("[Info] Copied size (size on disk):  %.0f (%.0f MB)\n"), ( float ) disk_vob_size, ( float ) disk_vob_size / ( 1024*1024 ) );
-
   if ( ( vob_size - disk_vob_size ) > MAX_DIFFER )
     {
       fprintf( stderr, _("[Error] Hmm, the sizes differ by more than %d\n"), MAX_DIFFER );
@@ -2026,14 +2049,8 @@ off_t get_used_space( char *path, int verbosity_level )
 int make_output_path( char *pwd,char *name,int get_dvd_name_return, char *dvd_name,int titleid, int partcount )
 {
   char temp[5];
-  int i;
   strcpy( name, pwd );
-  if( !get_dvd_name_return )
-    {
-      strcat( name, dvd_name );
-    }
-  else
-    strcat( name, "the_video_you_wanted" );
+  strcat( name, dvd_name );
 
   sprintf( temp, "%d", titleid );
   strcat( name, temp );
@@ -2045,13 +2062,6 @@ int make_output_path( char *pwd,char *name,int get_dvd_name_return, char *dvd_na
     }
   strcat( name, ".vob" );
 
-  for( i=0; i< strlen(name);i++ )
-    {
-      if( name[i] == ' ')
-        name[i] = '_';
-    }
-
-
   fprintf( stderr, _("\n[Info] Outputting to %s"), name );
   return 0;
 }
@@ -2062,9 +2072,12 @@ int make_output_path( char *pwd,char *name,int get_dvd_name_return, char *dvd_na
 
 void usage( char *program_name )
 {
-  fprintf( stderr, _("Vobcopy "VERSION" - GPL Copyright (c) 2001 - 2008 robos@muon.de\n") );
+  fprintf( stderr, _("Vobcopy "VERSION" - GPL Copyright (c) 2001 - 2009 robos@muon.de\n") );
   fprintf( stderr, _("\nUsage: %s \n"), program_name );
-  fprintf( stderr, _("[-m (mirror)] \n") );
+  fprintf( stderr, _("if you want the main feature (title with most chapters) you don't need _any_ options!\n") );
+  fprintf( stderr, _("Options:\n") );
+  fprintf( stderr, _("[-m (mirror the whole dvd)] \n") );
+  fprintf( stderr, _("[-M (Main title - i.e. the longest (playing time) title on the dvd)] \n") );
   fprintf( stderr, _("[-i /path/to/the/mounted/dvd/]\n") );
   fprintf( stderr, _("[-n title-number] \n") );
   fprintf( stderr, _("[-t <your name for the dvd>] \n") );
@@ -2075,7 +2088,7 @@ void usage( char *program_name )
   fprintf( stderr, _("[-v -v (create log-file)]\n") );
   fprintf( stderr, _("[-h (this here ;-)] \n") );
   fprintf( stderr, _("[-I (infos about title, chapters and angles on the dvd)]\n") );
-  fprintf( stderr, _("[-1/path/to/second/output/dir/] [-2/.../third/..] [-3/../] [-4 /../]\n") );
+  fprintf( stderr, _("[-1 /path/to/second/output/dir/] [-2 /.../third/..] [-3 /../] [-4 /../]\n") );
   fprintf( stderr, _("[-b <skip-size-at-beginning[bkmg]>] \n") );
   fprintf( stderr, _("[-e <skip-size-at-end[bkmg]>]\n") );
   fprintf( stderr, _("[-O <single_file_name1,single_file_name2, ...>] \n") );
@@ -2204,6 +2217,63 @@ int makedir( char *name )
   return 0;
 }
 
+/*
+* Check the time determine whether a new progress line should be output (once per second)
+*/
+
+int progressUpdate(int starttime, int cur, int tot, int force)
+{
+  static int progress_time = 0;
+
+  if (progress_time == 0 || progress_time != time(NULL) || force)
+  {
+	  int barLen, barChar, numChars, timeSoFar, minsLeft, secsLeft, ctr, cols;
+	  float percentComplete, percentLeft, timePerPercent;
+	  int curtime, timeLeft;
+	  struct winsize ws; 
+
+	  ioctl(0, TIOCGWINSZ, &ws);
+	  cols = ws.ws_col - 1;
+
+	  progress_time = time(NULL);
+	  curtime = time(NULL);
+
+	  printf("\r");
+/* 	   calc it this way so it's easy to change later */
+/* 	   2 for brackets, 1 for space, 5 for percent complete, 1 for space, 6 for time left, 1 for space */
+	  barLen = cols - 2 - 1 - 5 - 1 - 6 - 1;
+	  barChar = tot / barLen;
+	  percentComplete = (float)((float)cur / (float)tot * 100.0);
+	  percentLeft = 100 - percentComplete;
+	  numChars = cur / barChar;
+
+/* 	   guess remaining time */
+	  timeSoFar = curtime - starttime;
+	  if (percentComplete == 0) {
+		  timePerPercent = 0;
+	  } else {
+		  timePerPercent = (float)(timeSoFar / percentComplete);
+	  }
+	  timeLeft = timePerPercent * percentLeft;
+	  minsLeft = (int)(timeLeft / 60);
+	  secsLeft = (int)(timeLeft % 60);
+
+	  printf("[");
+	  for (ctr = 0; ctr < numChars-1; ctr++) {
+		  printf("=");
+	  }
+	  printf("|");
+	  for (ctr = numChars; ctr < barLen; ctr++) {
+		  printf(" ");
+	  }
+	  printf("] ");
+	  printf("%5.1f%% %02d:%02d ", percentComplete, minsLeft, secsLeft);
+	  fflush(stdout);
+  }
+  return(0);
+}
+
+
 void install_signal_handlers()
 {
   struct sigaction action;
@@ -2244,3 +2314,20 @@ int fdatasync( int value )
   return 0;
 }
 #endif
+
+
+/*
+* Check the time determine whether a new progress line should be output (once per second)
+*/
+
+int check_progress()
+{
+  static int progress_time = 0;
+
+  if ( progress_time == 0 || progress_time != time(NULL) )
+    {
+      progress_time = time(NULL);
+      return( 1 );
+    }
+  return( 0 );
+}
